<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Checker\Rule;

use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;
use Sylius\Component\Core\OrderPaymentStates;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Webmozart\Assert\Assert;

final class NumberOfOrdersRuleChecker implements BonusPointsStrategyRuleCheckerInterface
{
    private OrderRepositoryInterface $orderRepository;

    public function __construct(OrderRepositoryInterface $orderRepository)
    {
        $this->orderRepository = $orderRepository;
    }

    public function isEligible(OrderItemInterface $orderItem, array $configuration): bool
    {
        Assert::keyExists($configuration, 'number');

        /** @var OrderInterface $order */
        $order = $orderItem->getOrder();
        Assert::isInstanceOf($order, OrderInterface::class);

        /** @var CustomerInterface $customer */
        $customer = $order->getCustomer();
        if (!$customer instanceof CustomerInterface) {
            return true;
        }

        $countPayedOrders = count($this->orderRepository->findBy([
            'customer' => $customer,
            'paymentState' => OrderPaymentStates::STATE_PAID,
        ]));

        // The payment completion triggering this rule checking has not been persisted yet so the order is counted as paid
        // We add 1 to the count to include it in the check
        return ($countPayedOrders + 1) > $configuration['number'];
    }
}
