<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\DependencyInjection\Compiler;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;

class RegisterBonusPointsEventsPass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container)
    {
        $bonusPointsExtensionConfiguration = $container->getExtensionConfig('bit_bag_sylius_bonus_points');

        $events = [];
        if (!empty($bonusPointsExtensionConfiguration) && is_array($bonusPointsExtensionConfiguration[0]) && array_key_exists('listeners', $bonusPointsExtensionConfiguration[0])) {
            $listeners = $bonusPointsExtensionConfiguration[0]['listeners'];

            foreach ($listeners as $listenerName => $config) {
                $container->findDefinition('bitbag_sylius_bonus_points.event_listener.bonus_points_event')->addTag('kernel.event_listener', [
                    'event' => $config['event'],
                    'priority' => $config['priority'],
                ]);

                $events[$config['calculator_type']][$config['event']] = $config['label'];
            }
        }

        $container->setParameter('bitbag.bonus_points_strategy_events', $events);
    }
}
